using System.Collections.Generic;
using Atomia.Billing.Core.Sdk;
using Atomia.Billing.Core.Sdk.Plugins;
using Atomia.Billing.Core.Sdk.Utils.Email;

namespace AtomiaBillingSdkExample.Plugins
{
    /// <summary>
    /// Custom implementation of IScheduledEventHandler.
    /// </summary>
    public class NotificationHandler : IScheduledEventHandler
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="NotificationHandler"/> class. 
        /// </summary>
        public NotificationHandler()
        {
            this.SenderAddress = "test@atomia.com";
        }

        /// <summary>
        /// Gets or sets plugin name.
        /// </summary>
        public string Name
        {
            get
            {
                return "NotificationHandler";
            }

            set
            {
            }
        }

        /// <summary>
        /// Gets or sets IAtomiaBillingApi instance.
        /// </summary>
        public IAtomiaBillingApi AtomiaBillingApi
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets SenderAddress.
        /// </summary>
        public string SenderAddress
        {
            get;
            set;
        }

        /// <summary>
        /// Handles scheduled event.
        /// </summary>
        public void HandleEvent()
        {
            IList<MyNotification> notifications = this.GetNotifications();
            foreach (MyNotification notification in notifications)
            {
                this.SendEmail(this.SenderAddress, notification.To, notification.Subject, notification.Body);
            }
        }

        /// <summary>
        /// Send email to recipient.
        /// </summary>
        /// <param name="senderAddress">The sender address.</param>
        /// <param name="recipient">The recipient.</param>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The email body.</param>
        private void SendEmail(string senderAddress, string recipient, string subject, string body)
        {
            this.AtomiaBillingApi.SendMail(senderAddress, recipient, subject, body, new List<MailAttachment>());
        }

        /// <summary>
        /// Gets notifications.
        /// </summary>
        /// <returns>
        /// List of notifications.
        /// </returns>
        private IList<MyNotification> GetNotifications()
        {
            IList<MyNotification> notifications = new List<MyNotification>();

            // Here should be placed logic for retrieving notifications (using helper class, repository, etc.).
            // For this example, one notification is hardcoded.
            notifications.Add(new MyNotification
            {
                Body = "Test notification body.",
                Subject = "Test notification",
                To = "info@foobar.com"
            });
            return notifications;
        }
    }

    /// <summary>
    /// Notification class.
    /// </summary>
    internal class MyNotification
    {
        /// <summary>
        /// Gets or sets the body.
        /// </summary>
        public string Body { get; set; }

        /// <summary>
        /// Gets or sets the subject.
        /// </summary>
        public string Subject { get; set; }

        /// <summary>
        /// Gets or sets recipient.
        /// </summary>
        public string To { get; set; }
    }
}