﻿using System.Collections.Generic;
using Atomia.Billing.Core.Sdk;
using Atomia.Billing.Core.Sdk.Utils.Email;
using AtomiaBillingSdkExample.Plugins;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;

namespace AtomiaBillingSdkExample.UnitTests
{
    /// <summary>
    /// NotificationHandler test fixture.
    /// </summary>
    [TestClass]
    public class NotificationHandlerTest
    {
        /// <summary>
        /// Rhino Mocks Repository.
        /// </summary>
        private readonly MockRepository mocks;

        /// <summary>
        /// Atomia Billing Api mock.
        /// </summary>
        private readonly IAtomiaBillingApi atomiaBillingApiMock;

        /// <summary>
        /// Initializes a new instance of the <see cref="NotificationHandlerTest"/> class. 
        /// </summary>
        public NotificationHandlerTest()
        {
            this.mocks = new MockRepository();
            this.atomiaBillingApiMock = this.mocks.StrictMock<IAtomiaBillingApi>();
        }

        /// <summary>
        /// Gets or sets the test context which provides
        /// information about and functionality for the current test run.
        /// </summary>
        public TestContext TestContext { get; set; }

        #region Additional test attributes
        /// <summary>
        /// Use ClassInitialize to run code before running the first test in the class.
        /// </summary>
        /// <param name="testContext">The test context.</param>
        [ClassInitialize]
        public static void MyClassInitialize(TestContext testContext)
        {
        }

        /// <summary>
        /// Use ClassCleanup to run code after all tests in a class have run.
        /// </summary>
        [ClassCleanup]
        public static void MyClassCleanup()
        {
        }

        /// <summary>
        /// Use TestInitialize to run code before running each test.
        /// </summary>
        [TestInitialize]
        public void MyTestInitialize()
        {
            this.mocks.BackToRecordAll();
        }

        /// <summary>
        /// Use TestCleanup to run code after each test has run.
        /// </summary>
        [TestCleanup]
        public void MyTestCleanup()
        {
            this.mocks.VerifyAll();
        }
        #endregion

        /// <summary>
        /// A test for HandleEvent
        /// </summary>
        [TestMethod]
        public void HandleEventTest()
        {
            NotificationHandler notificationHandler = new NotificationHandler
            {
                AtomiaBillingApi = this.atomiaBillingApiMock
            };

            Expect.Call(
                () =>
                this.atomiaBillingApiMock.SendMail("test@atomia.com", "info@foobar.com", "Test notification", "Test notification body.", new List<MailAttachment>()))
                .Repeat.Once();
            this.mocks.ReplayAll();

            notificationHandler.HandleEvent();
        }
    }
}